"""This module contains the Enemy class."""
import pygame as pg
import math


class Enemy:
    """Class for the enemy object."""
    def __init__(self, hp: float, speed: float, damage: float, full_path_time: float, texture_path: str):
        """Initializes the enemy object."""
        self.max_hp = hp
        self.hp = self.max_hp
        self.speed = speed
        self.damage = damage
        self.texture_path = texture_path
        self.texture = pg.image.load(self.texture_path).convert_alpha()
        self.path_position = 0
        self.screen_position = (0, 0)
        self.full_path_time = full_path_time
        self.render_size = 64
        self.reward_on_death = 100

    def calculate_current_path_segment(self, path: tuple[tuple[int, int]]) -> tuple[int, float]:
        """
        Returns the index of current path segment and the distance along the path segment.

        Parameters:
            waypoints:
                The waypoints of the path.
        """
        # First find the length of each path segment
        path_lengths = []
        for i in range(len(path) - 1):
            path_lengths.append(((path[i][0] - path[i + 1][0])**2 + (path[i][1] - path[i + 1][1])**2) ** 0.5)
        full_path_length = sum(path_lengths)

        # Then find the current path segment and the distance along it
        current_path_length = 0
        for i in range(len(path_lengths)):
            if self.path_position < (current_path_length + path_lengths[i]) / full_path_length:
                segment_position = (self.path_position * full_path_length - current_path_length) / path_lengths[i]
                return i, segment_position
            current_path_length += path_lengths[i]
        if current_path_length >= full_path_length:
            # Return the end of the last path segment if the path_position more than 1
            return len(path) - 2, 1
        
    def move_along_path(self, dt: float, path: tuple[tuple[int, int]]):
        """
        Moves the enemy along the path.

        Parameters:
            dt:
                The time passed since the last frame.
            path:
                The waypoints of the path.
        """
        self.path_position += self.speed * dt / self.full_path_time
        #This is to check if the zombies have reached the end of the path for the health and destrcution purposes
        if self.path_position >= 1:
            return True
        path_segment, segment_position = self.calculate_current_path_segment(path)
        self.screen_position = (
            path[path_segment][0] + (path[path_segment + 1][0] - path[path_segment][0]) * segment_position,
            path[path_segment][1] + (path[path_segment + 1][1] - path[path_segment][1]) * segment_position
        )
    
    def face_movement_direction(self, path):
        """
        Rotates the enemy's texture to face the direction of movement.

        Parameters:
            path:
                The waypoints of the path.
        """

        path_segment, not_used = self.calculate_current_path_segment(path)
        current_segment_start = path[path_segment]
        current_segment_end = path[path_segment + 1]

        dx = current_segment_end[0] - current_segment_start[0]
        dy = current_segment_end[1] - current_segment_start[1]
        angle = -math.degrees(math.atan2(dy, dx))
        self.rotated_texture = pg.transform.rotozoom(self.texture, angle, 1)
    
    def generate_health_bar(self) -> pg.Surface:
        health_bar = pg.Surface((self.render_size, 8))
        health_bar.fill((255, 0, 0))
        health_bar.fill((0, 255, 0), (0, 0, self.render_size * (self.hp / self.max_hp), 8))
        return health_bar
    
    def draw(self, surface, path: tuple[tuple[int, int]]):
        """
        Draws the enemy to the passed surface using the rotated texture
        and displays the health bar under the enemy.

        Parameters:
            surface:
                The surface to draw the enemy to.
            path:
                The waypoints of the path.
        """
        self.face_movement_direction(path)
        self.texture_rect = self.rotated_texture.get_rect()
        self.texture_rect.center = self.screen_position
        surface.blit(self.rotated_texture, self.texture_rect)

        # Generate health bar
        health_bar = self.generate_health_bar()
        health_bar_rect = health_bar.get_rect()
        # Position the health bar under the enemy
        health_bar_rect.centerx = self.texture_rect.centerx
        health_bar_rect.top = self.texture_rect.bottom + 2
        # Blit the health bar onto the passed surface
        surface.blit(health_bar, health_bar_rect)


class Zombie(Enemy):
    """Class for the zombie object."""
    def __init__(self, hp: float):
        """Initializes the zombie object."""
        super().__init__(hp, 1, 10, 30, 'assets/images/zombie/zombie_128x128.png')
